/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Mozilla.org.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Dan Yamaoka
 *
 * ***** END LICENSE BLOCK ***** */

/*----------------------
 CookieCuller 1.02
 Dan Yamaoka
 Original CookieManager code is from Mozilla Firefox v0.8, CookieViewer.js
 Modified original code is commented with "//DY" (I hope)
 
 History:
 1.00 Original release for Firefox v0.8
 1.01 No code changed
 1.02 Updated for Firefox v0.9:removed cookie Policy & Status props,changed Secure strings
 ----------------------*/

// interface variables
var cookiemanager = Components.classes["@mozilla.org/cookiemanager;1"].getService();
    cookiemanager = cookiemanager.QueryInterface(Components.interfaces.nsICookieManager);
var gDateService = null;

// cookies list
var cookies              = [];
var deletedCookies       = [];

var cookieBundle;
var cookiesTree;
const nsICookie = Components.interfaces.nsICookie;

//DY - cookieculler globals
var savedcookies         = [];
var cookieculler_prefs = Components.classes["@mozilla.org/preferences-service;1"].
                             getService(Components.interfaces.nsIPrefService)
                             .getBranch("extensions.cookieculler.");
const cookiepref = "pcookie";
const cookieprefdelimiter = ";";

function initDialog() {
  // intialize gDateService
  if (!gDateService) {
    const nsScriptableDateFormat_CONTRACTID = "@mozilla.org/intl/scriptabledateformat;1";
    const nsIScriptableDateFormat = Components.interfaces.nsIScriptableDateFormat;
    gDateService = Components.classes[nsScriptableDateFormat_CONTRACTID]
      .getService(nsIScriptableDateFormat);
  }

  //XXXBlake
  // I removed the observer stuff, so yes, there are edge cases where
  // if you're loading a page when you open prefs/cookie manager,
  // the cookie manager won't display the new cookie added.
  // I don't think it's a big deal (considering how hard it would be to fix)

//DY - load saved protected cookies
  loadProtCookies();

//DY - load saved cookies
  loadCookies();
}

function onOK() {
//DY - commented out these two funcs--our dialog doesn't use PrefPanel system
//  window.opener.top.wsm.savePageData(window.location.href, window);
//  window.opener.top.hPrefWindow.registerOKCallbackFunc(window.opener.cookieViewerOnPrefsOK);

// Save user changes before exit
  FinalizeCookieDeletions();

//DY - Save protected Cookies before exit
  saveProtCookies();
}

//DY - These two funcs used by PrefPanel system; replaced with initDialog() code and XUL persist code
/*
function GetFields()
{
 var dataObject = {};
  dataObject.deletedCookies = deletedCookies;
  dataObject.cookies = cookies;
  dataObject.cookieBool = document.getElementById("checkbox").checked;
  return dataObject;
}

function SetFields(dataObject)
{
  if ('cookies' in dataObject)
    cookies = dataObject.cookies;

  if ('deletedCookies' in dataObject)
    deletedCookies = dataObject.deletedCookies;
  
  if ('cookieBool' in dataObject)
    document.getElementById("checkbox").checked = dataObject.cookieBool;

  loadCookies();
}
*/

/*** =================== COOKIES CODE =================== ***/

//DY - added lockedCol handling
var cookiesTreeView = {
  rowCount : 0,
  setTree : function(tree){},
  getImageSrc : function(row,column) {},
  getProgressMode : function(row,column) {},
  getCellValue : function(row,column) {},
  getCellText : function(row,column){
    var rv="";
    switch (column) {
      case "domainCol" : rv = cookies[row].rawHost; break;
      case "nameCol"   : rv = cookies[row].name; break;
      case "lockCol"   : rv = cookies[row].isProtected ? cookieBundle.getString("yes") : cookieBundle.getString("no"); break;
    }
    return rv;
  },
  isSeparator : function(index) {return false;},
  isSorted: function() { return false; },
  isContainer : function(index) {return false;},
  cycleHeader : function(aColId, aElt) {},
  getRowProperties : function(row,column,prop){},
  getColumnProperties : function(column,columnElement,prop){},
  getCellProperties : function(row,prop){}
 };

//DY - Added 'protected' attribute
function Cookie(number,name,value,isDomain,host,rawHost,path,isSecure,expires,
                isProtected) {
  this.number = number;
  this.name = name;
  this.value = value;
  this.isDomain = isDomain;
  this.host = host;
  this.rawHost = rawHost;
  this.path = path;
  this.isSecure = isSecure;
  this.expires = expires;
//DY - Removed for v0.9+ compatibility
//  this.status = status;
//  this.policy = policy;
  this.isProtected = isProtected;
}

function loadCookies() {
  if (!cookieBundle)
//DY
//    cookieBundle = document.getElementById("cookieBundle");
    cookieBundle = document.getElementById("cookiecullerBundle");

  if (!cookiesTree)
    cookiesTree = document.getElementById("cookiesTree");

//DY - No PrefPanel Support
//  var dataObject = window.opener.top.hPrefWindow.wsm.dataManager.pageData[window.location.href].userData;
//  if (!('cookies' in dataObject)) {
    // load cookies into a table
    var enumerator = cookiemanager.enumerator;
    var count = 0;
    while (enumerator.hasMoreElements()) {
      var nextCookie = enumerator.getNext();
      if (!nextCookie) break;
      nextCookie = nextCookie.QueryInterface(Components.interfaces.nsICookie);
      var host = nextCookie.host;
//DY - Removed for v0.9+ compatibility
//      if (nextCookie.policy != nsICookie.POLICY_UNKNOWN) {
//        showPolicyField = true;
//      }

//DY - Check if cookie is protected
      var isProt = checkIfProtected(nextCookie.name, host, nextCookie.path);

      cookies[count] =
        new Cookie(count++, nextCookie.name, nextCookie.value, nextCookie.isDomain, host,
                   (host.charAt(0)==".") ? host.substring(1,host.length) : host,
                   nextCookie.path, nextCookie.isSecure, nextCookie.expires,
                   isProt);

    }

  cookiesTreeView.rowCount = cookies.length;
  cookiesTree.treeBoxObject.view = cookiesTreeView;

 // sort by host column
  CookieColumnSort('rawHost');

  //disable "remove all cookies" button if there are no cookies
  document.getElementById("removeAllCookies").disabled = cookies.length == 0;
}

function GetExpiresString(expires) {
  if (expires) {
    var date = new Date(1000*expires);
    return gDateService.FormatDateTime("", gDateService.dateFormatLong,
                                       gDateService.timeFormatSeconds, date.getFullYear(),
                                       date.getMonth()+1, date.getDate(), date.getHours(),
                                       date.getMinutes(), date.getSeconds());
  }
  return cookieBundle.getString("AtEndOfSession");
}

//DY - Removed for v0.9+ compatibility
/*
function GetPolicyString(policy) {
  switch (policy) {
    case nsICookie.POLICY_NONE:
      return cookieBundle.getString("policyUnstated");
    case nsICookie.POLICY_NO_CONSENT:
      return cookieBundle.getString("policyNoConsent");
    case nsICookie.POLICY_IMPLICIT_CONSENT:
      return cookieBundle.getString("policyImplicitConsent");
    case nsICookie.POLICY_EXPLICIT_CONSENT:
      return cookieBundle.getString("policyExplicitConsent");
    case nsICookie.POLICY_NO_II:
      return cookieBundle.getString("policyNoIICollected");
  }
  return "";
}
*/

function CookieSelected() {
  var selections = GetTreeSelections(cookiesTree);
  if (selections.length) {

//DY - check if (the last in list) selection is protected/unprotected, set buttons
    if (cookies[selections[(selections.length)-1]].isProtected) {
        document.getElementById("removeCookie").disabled = true;
        document.getElementById("unprotectCookie").disabled = false;
        document.getElementById("protectCookie").disabled = true;
    } else {
        document.getElementById("removeCookie").disabled = false;
        document.getElementById("unprotectCookie").disabled = true;
        document.getElementById("protectCookie").disabled = false;
    }

  } else {

    ClearCookieProperties();

//DY - disable Delete & Protection buttons if selected cookie is unselected
    document.getElementById("removeCookie").disabled = true;
    document.getElementById("unprotectCookie").disabled = true;
    document.getElementById("protectCookie").disabled = true;

    return true;
  }
    
  var idx = selections[0];
  if (idx >= cookies.length) {
    // Something got out of synch.  See bug 119812 for details
    dump("Tree and viewer state are out of sync! " +
         "Help us figure out the problem in bug 119812");
    return false;
  }

  var props = [
    {id: "ifl_name", value: cookies[idx].name},
    {id: "ifl_value", value: cookies[idx].value}, 
    {id: "ifl_isDomain",
     value: cookies[idx].isDomain ?
            cookieBundle.getString("domainColon") : cookieBundle.getString("hostColon")},
    {id: "ifl_host", value: cookies[idx].host},
    {id: "ifl_path", value: cookies[idx].path},
    {id: "ifl_isSecure",
     value: cookies[idx].isSecure ?
//DY - New strings for v0.9+ compatibility
//            cookieBundle.getString("yes") : cookieBundle.getString("no")},
            cookieBundle.getString("forSecureOnly") : 
            cookieBundle.getString("forAnyConnection")},
    {id: "ifl_expires", value: GetExpiresString(cookies[idx].expires)},
//DY - Removed for v0.9+ compatibility
//    {id: "ifl_policy", value: GetPolicyString(cookies[idx].policy)}
  ];

  var value;
  var field;
  for (var i = 0; i < props.length; i++)
  {
    field = document.getElementById(props[i].id);
    if ((selections.length > 1) && (props[i].id != "ifl_isDomain")) {
      value = ""; // clear field if multiple selections
    } else {
      value = props[i].value;
    }
    field.value = value;
  }
  return true;
}

function ClearCookieProperties() {
  var properties = 
//DY - Remove unused prop for v0.9+ compatibility
//    ["ifl_name","ifl_value","ifl_host","ifl_path","ifl_isSecure","ifl_expires","ifl_policy"];
    ["ifl_name","ifl_value","ifl_host","ifl_path","ifl_isSecure","ifl_expires"];
  for (var prop=0; prop<properties.length; prop++) {
    document.getElementById(properties[prop]).value = "";
  }
}

function DeleteCookie() {
//DY - check if any selection is protected
    var selections = GetTreeSelections(cookiesTree);
    var protect = false;
    var i;
    for (i=0; i<selections.length; i++) {
       if (cookies[selections[i]].isProtected) {
           protect = true;
       }
    }
    if (!protect && i>0 ) {
      DeleteSelectedItemFromTree(cookiesTree, cookiesTreeView,
                                  cookies, deletedCookies,
                                  "removeCookie", "removeAllCookies",
                                  "protectCookie", "unprotectCookie");
      if (!cookies.length) {
        ClearCookieProperties();
      }

    }

//DY - Move to onOK() so changes are cancellable
//  FinalizeCookieDeletions();
}

function DeleteAllCookies() {

  ClearCookieProperties();
  DeleteAllFromTree(cookiesTree, cookiesTreeView,
                        cookies, deletedCookies,
                        "removeCookie", "removeAllCookies",
                        "protectCookie", "unprotectCookie");

//DY - Move to onOK() so changes are cancellable
//  FinalizeCookieDeletions();
}

function FinalizeCookieDeletions() {
  for (var c=0; c<deletedCookies.length; c++) {
    cookiemanager.remove(deletedCookies[c].host,
                         deletedCookies[c].name,
                         deletedCookies[c].path,
                         document.getElementById("checkbox").checked);
  }
  deletedCookies.length = 0;
}

function HandleCookieKeyPress(e) {
  if (e.keyCode == 46) {
     DeleteCookie();
  }
}

var lastCookieSortColumn = "";
var lastCookieSortAscending = false;

function CookieColumnSort(column) {
  lastCookieSortAscending =
    SortTree(cookiesTree, cookiesTreeView, cookies,
                 column, lastCookieSortColumn, lastCookieSortAscending);
  lastCookieSortColumn = column;
}

//DY - All new functions below
function checkIfProtected(name, host, path) {
//return true if Cookie is protected
  var rv = false;
  for (var i=0; i<savedcookies.length; i++) {
    if ( (savedcookies[i].host == host) && (savedcookies[i].path == path) && (savedcookies[i].name == name) ) {
      rv = true;
      break;
    }
  }
  return rv;
}

//put saved protected cookie list in savedcookies[]
function loadProtCookies() {
  var savecount = 0;
  var truecount = 0;

  if (cookieculler_prefs.prefHasUserValue("count")) {
     savecount = cookieculler_prefs.getIntPref("count");
        for (var i=0; i<savecount; i++) {
           if (cookieculler_prefs.prefHasUserValue(cookiepref + i)) {
              var protectedcookie = cookieculler_prefs.getCharPref(cookiepref + i);
              var secondDelimiter = protectedcookie.lastIndexOf(cookieprefdelimiter);
              var cpath = protectedcookie.slice(secondDelimiter+1);
              var firstDelimiter = protectedcookie.lastIndexOf(cookieprefdelimiter,secondDelimiter-1);
              var cname = protectedcookie.slice(firstDelimiter+1, secondDelimiter);
              var chost = protectedcookie.slice(0,firstDelimiter);
              savedcookies[truecount] = new CookieProtected(truecount++, cname, chost, cpath);
           }
        }
  }

}

//build list of protected cookies from 'cookies' object and save in prefs
function saveProtCookies() {

  var savecount = 0;
//Remove all previously saved protected cookies to prevent out-of-sync problems
  cookieculler_prefs.deleteBranch("");
  
//prefs are stored in user's pref.js as "pcookie#" <-> "host;name;path"
  for (var i = 0; i < cookies.length; i++) {
     if (cookies[i].isProtected) {
         cookieculler_prefs.setCharPref(cookiepref + savecount, cookies[i].host + cookieprefdelimiter +
                      cookies[i].name + cookieprefdelimiter + cookies[i].path);
         savecount++;
     }
  }
  cookieculler_prefs.setIntPref("count", savecount);
}

function ProtectCookie() {
  ProtectInTree(cookiesTree, cookiesTreeView,
                        cookies, "protectCookie", "unprotectCookie", "removeCookie");
}

function UnprotectCookie() {
  UnProtectInTree(cookiesTree, cookiesTreeView,
                        cookies, "protectCookie", "unprotectCookie", "removeCookie");
}

function CookieProtected(number,name,host,path) {
  this.number = number;
  this.name = name;
  this.host = host;
  this.path = path;
}

